'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = createSource;

var _graphql = require('graphql');

var _graphqlTools = require('graphql-tools');

var _graphqlTypeJson = require('graphql-type-json');

var _graphqlTypeJson2 = _interopRequireDefault(_graphqlTypeJson);

var _graphqlIsoDate = require('graphql-iso-date');

var _changeCase = require('change-case');

var _normalizr = require('normalizr');

var _pluralize = require('pluralize');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }
// https://stackoverflow.com/questions/46562561/apollo-graphql-field-type-for-object-with-dynamic-keys

// https://www.npmjs.com/package/graphql-iso-date


var RE_LIST_SUFFIX = /_$list(_|$)/;

function astToNormalizeSchema(gqlAst, _ref) {
  var idAttribute = _ref.idAttribute;
  var definitions = gqlAst.definitions;

  var framents = {};
  definitions.forEach(function (definition) {
    var kind = definition.kind,
        name = definition.name,
        alias = definition.alias;

    if (kind === 'FragmentDefinition') {
      framents[alias ? alias.value : name.value] = definition;
    }
  });
  var normalizeSchema = {};
  /* eslint-disable max-statements */
  var traverseProp = function traverseProp(root) {
    return function (field) {
      var name = field.name,
          alias = field.alias,
          selectionSet = field.selectionSet;
      var originName = name.value;

      if (!selectionSet || originName === idAttribute) {
        return;
      }
      var spreadSections = [];
      selectionSet.selections.forEach(function (selection) {
        if (selection.kind === 'FragmentSpread') {
          var fragment = framents[selection.alias ? selection.alias.value : selection.name.value];
          if (!fragment || !fragment.selectionSet) {
            return;
          }
          spreadSections.push.apply(spreadSections, _toConsumableArray(fragment.selectionSet.selections));
        } else if (selection.kind === 'Field') {
          spreadSections.push(selection);
        }
      });
      var hasId = false;
      for (var i = 0; i < spreadSections.length; i++) {
        if (spreadSections[i].name.value === idAttribute) {
          hasId = true;
          break;
        }
      }
      // if (!hasId && normalizeSchema === root) {
      //   throw new Error(
      //     '[redux-source] The result of query/mutation operation (an object or objects in an array) must have id attrubute',
      //   );
      // }
      var childRoot = {};
      spreadSections.forEach(traverseProp(childRoot));
      var isList = false;
      if (RE_LIST_SUFFIX.test(originName)) {
        isList = true;
        originName = originName.replace(RE_LIST_SUFFIX, '$1');
        name.value = originName;
      } else if ((0, _pluralize.isPlural)(originName)) {
        isList = true;
      }
      var dataName = alias ? alias.value : originName;
      var entity = hasId ? new _normalizr.schema.Entity((0, _pluralize.singular)(dataName), childRoot, {
        idAttribute: idAttribute
      }) : childRoot;
      root[dataName] = isList ? [entity] : entity;
    };
  };
  /* eslint-enable max-statements */
  definitions.forEach(function (definition) {
    var kind = definition.kind,
        selectionSet = definition.selectionSet;

    if (kind !== 'OperationDefinition') {
      return;
    }
    selectionSet.selections.forEach(traverseProp(normalizeSchema));
  });
  return normalizeSchema;
}

function getConfig(selectionSet) {
  var config = {};
  var selections = selectionSet && selectionSet.selections || [];

  var _ref2 = selections[0] || {},
      configFieldName = _ref2.name.value,
      configSelectionSet = _ref2.selectionSet;

  if (configFieldName === '__config__') {
    selections.shift();
    if (configSelectionSet) {
      configSelectionSet.selections.forEach(function (_ref3) {
        var key = _ref3.alias.value,
            value = _ref3.name.value;

        config[key] = value;
      });
    }
  }
  return config;
}

function createSource(_ref4) {
  var typeDefs = _ref4.typeDefs,
      resolvers = _ref4.resolvers,
      createInitialState = _ref4.createInitialState,
      pendingReducer = _ref4.pendingReducer,
      successReducer = _ref4.successReducer,
      errorReducer = _ref4.errorReducer;

  var executableSchema = (0, _graphqlTools.makeExecutableSchema)({
    // https://github.com/apollographql/graphql-tools/blob/master/docs/source/scalars.md
    typeDefs: '\n      scalar JSON\n      scalar Date\n      scalar Time\n      scalar DateTime\n      ' + typeDefs + '\n    ',
    resolvers: _extends({
      JSON: _graphqlTypeJson2.default,
      Date: _graphqlIsoDate.GraphQLDate,
      Time: _graphqlIsoDate.GraphQLTime,
      DateTime: _graphqlIsoDate.GraphQLDateTime
    }, resolvers)
  });
  return function source(gqlAst) {
    var opt = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var _opt$stateName = opt.stateName,
        stateName = _opt$stateName === undefined ? 'source' : _opt$stateName,
        _opt$idAttribute = opt.idAttribute,
        idAttribute = _opt$idAttribute === undefined ? 'id' : _opt$idAttribute,
        _opt$namespace = opt.namespace,
        namespace = _opt$namespace === undefined ? 'REDUX_SOURCE' : _opt$namespace,
        _opt$delimiter = opt.delimiter,
        delimiter = _opt$delimiter === undefined ? '/' : _opt$delimiter;
    // console.log'GQL AST', gqlAst);

    var definitions = gqlAst.definitions;

    var actions = {};
    var reducerMap = {};
    var initialState = createInitialState({
      stateName: stateName
    });
    definitions.forEach(function (_ref5) {
      var name = _ref5.name,
          kind = _ref5.kind,
          operation = _ref5.operation,
          selectionSet = _ref5.selectionSet;

      if (kind !== 'OperationDefinition') {
        return;
      }
      var config = getConfig(selectionSet);
      var TYPE = [namespace, '' + (0, _changeCase.constantCase)(name.value)].join(delimiter);
      var PENDING_TYPE = TYPE + '_PENDING';
      var SUCCESS_TYPE = TYPE + '_SUCCESS';
      var ERROR_TYPE = TYPE + '_ERROR';
      var actionCreator = function actionCreator(args) {
        return function (dispatch) {
          dispatch({
            type: PENDING_TYPE
          });
          (0, _graphql.execute)(executableSchema, gqlAst, null, {}, args, name.value).then(function (result) {
            if (result.errors && result.errors.length) {
              return dispatch({
                type: ERROR_TYPE,
                payload: result.errors
              });
            }
            return dispatch({
              type: SUCCESS_TYPE,
              payload: result.data ? (0, _normalizr.normalize)(result.data, normalizeSchema) : null
            });
          });
        };
      };
      actions[TYPE] = actionCreator;
      reducerMap[PENDING_TYPE] = pendingReducer({
        stateName: stateName,
        operationName: name.value,
        operationType: operation,
        config: config
      });
      reducerMap[SUCCESS_TYPE] = successReducer({
        stateName: stateName,
        operationName: name.value,
        operationType: operation,
        config: config
      });
      reducerMap[ERROR_TYPE] = errorReducer({
        stateName: stateName,
        operationName: name.value,
        operationType: operation,
        config: config
      });
    });
    var normalizeSchema = astToNormalizeSchema(gqlAst, { idAttribute: idAttribute });
    return {
      actions: actions,
      reducerMap: reducerMap,
      initialState: initialState,
      normalizeSchema: normalizeSchema,
      stateName: stateName,
      idAttribute: idAttribute,
      namespace: namespace,
      delimiter: delimiter,
      normalize: function normalize(data) {
        return (0, _normalizr.normalize)(data, normalizeSchema);
      },
      denormalize: function denormalize(result, entities) {
        return (0, _normalizr.denormalize)(result, normalizeSchema, entities);
      }
    };
  };
}
module.exports = exports['default'];
//# sourceMappingURL=createSource.js.map